import React, { useEffect, useRef, useState } from 'react'
import { Box } from '@mui/material'
import { useTheme, type Theme } from '@mui/material/styles'
import {
  useLocalCameraTrack,
  useLocalMicrophoneTrack,
  useRemoteUsers,
  usePublish,
  useRTCClient,
  useClientEvent,
} from 'agora-rtc-react'
import SymmetricGrid from '../SymmetricGrid/SymmetricGrid'
import PinnedGrid from '../PinnedGrid/PinnedGrid'
import type { IAgoraRTCRemoteUser } from 'agora-rtc-sdk-ng'

const MeetingLayout = () => {
  const theme: Theme = useTheme()

  const agoraEngine = useRTCClient()
  const { localCameraTrack } = useLocalCameraTrack()
  const { localMicrophoneTrack } = useLocalMicrophoneTrack()
  usePublish([localMicrophoneTrack, localCameraTrack])
  const remoteUsers: IAgoraRTCRemoteUser[] = useRemoteUsers()

  const gridContainerRef = useRef<HTMLDivElement>(null)
  const [gridContainerHeight, setGridContainerHeight] = useState(0)

  const [totalUsers, setTotalUsers] = useState(0)
  const [isPinned, setIsPinned] = useState(false)
  const [pinnedUser, setPinnedUser] = useState<IAgoraRTCRemoteUser | null>(null)

  useEffect(() => {
    setTotalUsers(remoteUsers.length + 1)
  }, [remoteUsers])

  useEffect(() => {
    if (gridContainerRef.current) {
      setGridContainerHeight(gridContainerRef.current.offsetHeight)
    }
  }, [gridContainerRef?.current?.offsetHeight])

  const handleUserLeft = (user: IAgoraRTCRemoteUser) => {
    if (user.uid === pinnedUser?.uid) {
      setIsPinned(false)
      setPinnedUser(null)
    }
  }

  useClientEvent(agoraEngine, 'user-left', handleUserLeft)

  return (
    <Box
      sx={{
        width: '100%',
        height: '80%',
        backgroundColor: theme.palette.background.default,
        padding: '30px',
        display: 'flex',
        flexDirection: 'column',
      }}
    >
      <Box
        sx={{
          display: 'flex',
          flexDirection: 'row',
          width: '100%',
          height: '100%',
        }}
        ref={gridContainerRef}
      >
        {isPinned && pinnedUser ? (
          <PinnedGrid
            totalUsers={totalUsers}
            gridContainerHeight={gridContainerHeight}
            setIsPinned={setIsPinned}
            pinnedUser={pinnedUser}
            setPinnedUser={setPinnedUser}
          />
        ) : (
          <SymmetricGrid
            totalUsers={totalUsers}
            gridContainerHeight={gridContainerHeight}
            isPinned={isPinned}
            setIsPinned={setIsPinned}
            setPinnedUser={setPinnedUser}
          />
        )}
      </Box>
    </Box>
  )
}

export default MeetingLayout
